#include <QtGui>
#include <QtNetwork>

#include <stdlib.h>

#include "server.h"

Server::Server(QWidget *parent)
    : QDialog(parent)
{
    statusLabel = new QLabel;
    label1 = new QLabel;
    label2 = new QLabel;
    label3 = new QLabel;
    quitButton = new QPushButton(tr("Quit"));
    quitButton->setAutoDefault(false);

    history = new QTextEdit;

    tcpServer = new QTcpServer(this);
    if (!tcpServer->listen()) {
        QMessageBox::critical(this, tr("Echo Server"),
                              tr("Unable to start the server: %1.")
                              .arg(tcpServer->errorString()));
        close();
        return;
    }

    statusLabel->setText(tr("The server is running on port %1.\n"
                            "Run the Fortune Client example now.")
                         .arg(tcpServer->serverPort()));

    label1->setText(tr("Message1"));
    label2->setText(tr("Message2"));
    label3->setText(tr("Message3"));

    fortunes << tr("You've been leading a dog's life. Stay off the furniture.")
             << tr("You've got to think about tomorrow.")
             << tr("You will be surprised by a loud noise.")
             << tr("You will feel hungry again in another hour.")
             << tr("You might have mail.")
             << tr("You cannot kill time without injuring eternity.")
             << tr("Computers are not intelligent. They only think they are.");

    connect(quitButton, SIGNAL(clicked()), this, SLOT(close()));
    connect(tcpServer, SIGNAL(newConnection()), this, SLOT(connectToClient()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(quitButton);
    buttonLayout->addStretch(1);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(statusLabel);
    mainLayout->addWidget(label1);
    mainLayout->addWidget(label2);
    mainLayout->addWidget(label3);
    mainLayout->addLayout(buttonLayout);
    mainLayout->addWidget(history);
    setLayout(mainLayout);

    setWindowTitle(tr("Echo Server"));
}

void Server::connectToClient()
{
    // establich connection
    clientConnection = tcpServer->nextPendingConnection();
    connect(clientConnection, SIGNAL(disconnected()),
            clientConnection, SLOT(deleteLater()));
    connect(clientConnection, SIGNAL(readyRead()), this, SLOT(doEcho()));

    blockSize = 0;


}

void Server::doEcho()
{
    // setup read streams
    QDataStream in (clientConnection);
    in.setVersion(QDataStream::Qt_4_0);

    history->append(tr("in doEcho, blockSize %1, bytesAvailable %2")
                    .arg(blockSize).arg(clientConnection->bytesAvailable()));

    if (blockSize == 0) {
        if (clientConnection->bytesAvailable() < (int)sizeof(quint16))
            return;

        in >> blockSize;
    }

    if (clientConnection->bytesAvailable() < blockSize)
        return;

    QString message;
    in >> message;

    label1->setText(message);
    label2->setText(tr("Last Local Port: %1").arg(clientConnection->localPort()));
    history->append(message);

    history->append(tr("Sending message to %1").arg(clientConnection->peerPort()));

    QByteArray block;
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_0);
    out << (quint16)0;
    out << message;
    out.device()->seek(0);
    out << (quint16)(block.size() - sizeof(quint16));

    clientConnection->write(block);

    history->append(tr("end doEcho, blockSize %1, bytesAvailable %2")
                    .arg(blockSize).arg(clientConnection->bytesAvailable()));

    blockSize=0;

}
